//* nouns.cpp
//********************************************************************
//*
//*  NOUNS - Loads nouns from NOUNS.SRC to NOUNS.DCY and English
//*  irregular plurals from IRRPLU.SRC to IRRPLU.DCY for SWAHILI.
//*  Copyright (C) 1997-1998 Morris David Fried
//*
//*  This program is free software; you can redistribute it and/or
//*  modify it under the terms of the GNU General Public License
//*  as published by the Free Software Foundation; either version 2
//*  of the License, or (at your option) any later version.
//*
//*  This program is distributed in the hope that it will be useful,
//*  but WITHOUT ANY WARRANTY; without even the implied warranty of
//*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//*  GNU General Public License for more details.
//*
//*  You should have received a copy of the GNU General Public License
//*  along with this program; if not, write to the Free Software
//*  Foundation, Inc., 59 Temple Place - Suite 330,
//*  Boston, MA 02111-1307, USA.
//*
//*  I welcome any questions or comments about this software and I can
//*  be contacted at HAWKEYEOP@JUNO.COM or at 23 Lincoln Road North,
//*  Plainview, NY 11803, USA.
//*
//********************************************************************
//* file includes
#include <fstream.h>
#include <string.h>
#include <stdio.h>
//* global variables
  int nouns_load (void);
  int irreg_plu_load (void);
  int control_load (void);
  FILE * source;        /*  input source file  */
  FILE * dictionary;    /*  output dictionary file  */
  FILE * ctl_file;      /*  control file - i/o  */
  char input_buffer[81];
  int i, j;
  int record_n = 0;
  int record_i = 0;
//* Main Routine
main()
{
  int rc = 0;
//* executable code
  rc = nouns_load ();
  if (rc == 0)  rc = irreg_plu_load ();
  if (rc == 0)  rc = control_load ();
  return rc;
}
int nouns_load (void)
{
//* variables
  int rcn = 0;
  char swahili_hold[19] = {'\x00'};
  char tie_breaker_string[5];
  char plural_index_string[5];
  struct dict_entry
  {
    char swa_word[19];
    char eng_word[19];
    int short tie_breaker;
    int short plural_index;
    char grammar_flag;
    char plural_prefix[5];
  };
  struct dict_entry dict_noun;
//* executable code
  source = fopen ("NOUNS.SRC","r");
  if (source == NULL)
  {
    cout << "\n" << "Unable to open input file - NOUNS.SRC";
    return 16;
  }
  dictionary = fopen ("NOUNS.DCY","wb");
  if (dictionary == NULL)
  {
    cout << "\n" << "Unable to open output file - NOUNS.DCY";
    rcn = 16;
    goto cleanup2;
  }
loop:
  input_buffer[0] = '\x00';    /*  reset input buffer  */
  fgets (input_buffer,81,source);
  if ((input_buffer[0] == '\x00') || (input_buffer[0] == ' '))
  {
    cout << "\n" << record_n << " records successfully written";
    goto cleanup;
  }
  record_n++;
  i = 0;
  while (input_buffer[i] != ',')
  {
    if (i == 18)
    {
      cout << "\n" << "Swahili word in line " << record_n
           << " more than eighteen letters long";
      rcn = 12;
      goto cleanup;
    }
    dict_noun.swa_word[i] = input_buffer[i];
    i++;
  }
  for (j = i; j < 19; j++)
    dict_noun.swa_word[j] = '\x00';   /*  null out rest of word  */
  if (strcmp(dict_noun.swa_word,swahili_hold) <= 0)
  {
      cout << "\n" << "Swahili word in line " << record_n
           << " not in alphabetical order";
      rcn = 12;
      goto cleanup;
  }
  swahili_hold = dict_noun.swa_word;
  i++;        /*  space past first comma  */
  j = 0;      /*  reset field count       */
  while (input_buffer[i] != ',')
  {
    if (j == 18)
    {
      cout << "\n" << "English word in line " << record_n
           << " more than eighteen letters long";
      rcn = 12;
      goto cleanup;
    }
    dict_noun.eng_word[j] = input_buffer[i];
    i++;  j++;
  }
  for (; j < 19; j++)
    dict_noun.eng_word[j] = '\x00';   /*  null out rest of word  */
  i++;        /*  space past next comma  */
  j = 0;      /*  reset field count       */
  while (input_buffer[i] != ',')
  {
    if (j == 4)
    {
      cout << "\n" << "Swahili tie-breaker in line " << record_n
           << " more than four characters long";
      rcn = 12;
      goto cleanup;
    }
    tie_breaker_string[j] = input_buffer[i];
    i++;  j++;
  }
  if (j == 0)  dict_noun.tie_breaker = 0;    /*  default to 0  */
  else
  {
    for (; j < 5; j++)
      tie_breaker_string[j] = '\x00';  /* null out rest of string */
    dict_noun.tie_breaker = atoi(tie_breaker_string);
  }
  i++;        /*  space past next comma  */
  j = 0;      /*  reset field count       */
  while (input_buffer[i] != ',')
  {
    if (j == 4)
    {
      cout << "\n" << "English plural index in line " << record_n
           << " more than four characters long";
      rcn = 12;
      goto cleanup;
    }
    plural_index_string[j] = input_buffer[i];
    i++;  j++;
  }
  if (j == 0)  dict_noun.plural_index = -1;   /*  default to -1  */
  else
  {
    for (; j < 5; j++)
      plural_index_string[j] = '\x00';  /* null out rest of string */
    dict_noun.plural_index = atoi(plural_index_string);
  }
  i++;        /*  space past next comma  */
  if (input_buffer[i] == ',')  dict_noun.grammar_flag = ' ';
    else  if (input_buffer[i+1] == ',')
    {
      i++;    /*  space past grammar flag parameter  */
      dict_noun.grammar_flag = input_buffer[i];
    }
    else
    {
      cout << "\n" << "Grammar flag in line " << record_n
           << " more than one character long";
      rcn = 12;
      goto cleanup;
    }
  i++;        /*  space past next comma  */
  for (j = 0; j < 4; i++, j++)
    dict_noun.plural_prefix[j] = input_buffer[i];
  dict_noun.plural_prefix[4] = '\x00';
  fwrite (&dict_noun, 48, 1, dictionary);
  goto loop;    /*  go back to read next record  */
cleanup:
  fclose (source);
cleanup2:
  fclose (dictionary);
  return rcn;
}
int irreg_plu_load (void)
{
//* variables
  int rci = 0;
  char eng_irreg_plural [20];
//* executable code
  source = fopen ("IRRPLU.SRC","r");
  if (source == NULL)
  {
    cout << "\n" << "Unable to open input file - IRRPLU.SRC";
    return 16;
  }
  dictionary = fopen ("IRRPLU.DCY","wb");
  if (source == NULL)
  {
    cout << "\n" << "Unable to open output file - IRRPLU.DCY";
    rci = 16;
    goto cleanup2;
  }
loop:
  input_buffer[0] = '\x00';    /*  reset input buffer  */
  fgets (input_buffer,81,source);
  if ((input_buffer[0] == '\x00') || (input_buffer[0] == ' '))
  {
    cout << "\n" << record_i;
    cout << " irregular plurals successfully written";
    goto cleanup;
  }
  record_i++;
  i = 0;
  while (input_buffer[i] != ';')
  {
    if (i == 19)
    {
      cout << "\n" << "English irregular plural in line " << record_i
           << " more than nineteen letters long";
      rci = 12;
      goto cleanup;
    }
    eng_irreg_plural[i] = input_buffer[i];
    i++;
  }
  for (j = i; j < 20; j++)
    eng_irreg_plural[j] = '\x00';   /*  null out rest of word  */
  fwrite (eng_irreg_plural, 20, 1, dictionary);
  goto loop;    /*  go back to read next record  */
cleanup:
  fclose (source);
cleanup2:
  fclose (dictionary);
  return rci;
}
int control_load (void)
{
//* variables
  int rcc = 0;
  char version_constant[4] = "1.0";
  char date_constant[8] = "01/1998";
  struct control_record
  {
    char program_version[4];
    char version_date[8];
    int tot_swahili_words;
    int noun_dict_entries;
    int verb_dict_entries;
    int adjetc_dict_entries;
    int eng_irreg_plu_entries;
  };
  struct control_record control_data;
//* executable code
  ctl_file = fopen ("SWAHILI.CTL","rb");
  if (ctl_file == NULL)
  {
    cout << "\n" << "Building new control file - SWAHILI.CTL";
    cout << "\n" << "Run VERBS and ADJETC to complete file";
    control_data.program_version = version_constant;
    control_data.version_date = date_constant;
    control_data.tot_swahili_words = record_n;
    control_data.noun_dict_entries = record_n;
    control_data.verb_dict_entries =
      control_data.adjetc_dict_entries = 0;
    control_data.eng_irreg_plu_entries = record_i;
  }
  else
  {
    cout << "\n" << "Updating existing control file - SWAHILI.CTL";
    fread (&control_data, 32, 1, ctl_file);
    fclose (ctl_file);
    control_data.tot_swahili_words = record_n +
      control_data.verb_dict_entries +
      control_data.adjetc_dict_entries;
    control_data.noun_dict_entries = record_n;
    control_data.eng_irreg_plu_entries = record_i;
  }
  ctl_file = fopen ("SWAHILI.CTL","wb");
  fwrite (&control_data, 32, 1, ctl_file);
  fclose (ctl_file);
  return rcc;
}
