//* verbs.cpp
//********************************************************************
//*
//*  VERBS - Loads verbs from VERBS.SRC to VERBS.DCY for SWAHILI.
//*  Copyright (C) 1997 Morris David Fried
//*
//*  This program is free software; you can redistribute it and/or
//*  modify it under the terms of the GNU General Public License
//*  as published by the Free Software Foundation; either version 2
//*  of the License, or (at your option) any later version.
//*
//*  This program is distributed in the hope that it will be useful,
//*  but WITHOUT ANY WARRANTY; without even the implied warranty of
//*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//*  GNU General Public License for more details.
//*
//*  You should have received a copy of the GNU General Public License
//*  along with this program; if not, write to the Free Software
//*  Foundation, Inc., 59 Temple Place - Suite 330,
//*  Boston, MA 02111-1307, USA.
//*
//*  I welcome any questions or comments about this software and I can
//*  be contacted at HAWKEYEOP@JUNO.COM or at 23 Lincoln Road North,
//*  Plainview, NY 11803, USA.
//*
//********************************************************************
//* file includes
#include <fstream.h>
#include <string.h>
#include <stdio.h>
//* Main Routine
main()
{
//* variables
  FILE * source;        /*  input source file  */
  FILE * dictionary;    /*  output dictionary file  */
  FILE * ctl_file;      /*  control file - i/o  */
  char input_buffer[121];
  char swahili_hold[19] = {'\x00'};
  int i, j;
  int rc = 0, record = 0;
  char tie_breaker_string[5];
  char filler_string[5];
  struct dict_entry
  {
    char swa_word[19];
    char eng_word[19];
    int short tie_breaker;
    int short filler1;
    char grammar_flag;
    char eng_ing_form[19];
    char eng_s_form[19];
    char eng_past[19];
    char eng_participle[19];
    char filler2;
  };
  struct dict_entry dict_verb;
  struct control_record
  {
    char program_version[4];
    char version_date[8];
    int tot_swahili_words;
    int noun_dict_entries;
    int verb_dict_entries;
    int adjetc_dict_entries;
    int eng_irreg_plu_entries;
  };
  struct control_record control_data;
//* executable code
  source = fopen ("VERBS.SRC","r");
  if (source == NULL)
  {
    cout << "\n" << "Unable to open input file - VERBS.SRC";
    return 16;
  }
  dictionary = fopen ("VERBS.DCY","wb");
  if (dictionary == NULL)
  {
    cout << "\n" << "Unable to open output file - VERBS.DCY";
    rc = 16;
    goto cleanup2;
  }
loop:
  input_buffer[0] = '\x00';    /*  reset input buffer  */
  fgets (input_buffer,121,source);
  if ((input_buffer[0] == '\x00') || (input_buffer[0] == ' '))
  {
    cout << "\n" << record << " records successfully written";
    goto cleanup;
  }
  record++;
  i = 0;
  while (input_buffer[i] != ',')
  {
    if (i == 18)
    {
      cout << "\n" << "Swahili word in line " << record
           << " more than eighteen letters long";
      rc = 12;
      goto cleanup;
    }
    dict_verb.swa_word[i] = input_buffer[i];
    i++;
  }
  for (j = i; j < 19; j++)
    dict_verb.swa_word[j] = '\x00';   /*  null out rest of word  */
  if (strcmp(dict_verb.swa_word,swahili_hold) <= 0)
  {
      cout << "\n" << "Swahili word in line " << record
           << " not in alphabetical order";
      rc = 12;
      goto cleanup;
  }
  swahili_hold = dict_verb.swa_word;
  i++;        /*  space past first comma  */
  j = 0;      /*  reset field count       */
  while (input_buffer[i] != ',')
  {
    if (j == 18)
    {
      cout << "\n" << "English word in line " << record
           << " more than eighteen letters long";
      rc = 12;
      goto cleanup;
    }
    dict_verb.eng_word[j] = input_buffer[i];
    i++;  j++;
  }
  for (; j < 19; j++)
    dict_verb.eng_word[j] = '\x00';   /*  null out rest of word  */
  i++;        /*  space past next comma  */
  j = 0;      /*  reset field count       */
  while (input_buffer[i] != ',')
  {
    if (j == 4)
    {
      cout << "\n" << "Swahili tie-breaker in line " << record
           << " more than four characters long";
      rc = 12;
      goto cleanup;
    }
    tie_breaker_string[j] = input_buffer[i];
    i++;  j++;
  }
  if (j == 0)  dict_verb.tie_breaker = 0;    /*  default to 0  */
  else
  {
    for (; j < 5; j++)
      tie_breaker_string[j] = '\x00';  /* null out rest of string */
    dict_verb.tie_breaker = atoi(tie_breaker_string);
  }
  i++;        /*  space past next comma  */
  j = 0;      /*  reset field count      */
  while (input_buffer[i] != ',')
  {
    if (j == 4)
    {
      cout << "\n" << "Filler field in line " << record
           << " more than four characters long";
      rc = 12;
      goto cleanup;
    }
    filler_string[j] = input_buffer[i];
    i++;  j++;
  }
  if (j == 0)  dict_verb.filler1 = 0;         /*  default to 0  */
  else
  {
    for (; j < 5; j++)
      filler_string[j] = '\x00';  /* null out rest of string */
    dict_verb.filler1 = atoi(filler_string);
  }
  i++;        /*  space past next comma  */
  if (input_buffer[i] == ',')  dict_verb.grammar_flag = ' ';
    else  if (input_buffer[i+1] == ',')
    {
      dict_verb.grammar_flag = input_buffer[i];
      i++;    /*  space past grammar flag parameter  */
    }
    else
    {
      cout << "\n" << "Grammar flag in line " << record
           << " more than one character long";
      rc = 12;
      goto cleanup;
    }
  i++;        /*  space past next comma  */
  j = 0;      /*  reset field count      */
  while (input_buffer[i] != ',')
  {
    if (j == 18)
    {
      cout << "\n" << "English '-ing' form in line " << record
           << " more than eighteen letters long";
      rc = 12;
      goto cleanup;
    }
    dict_verb.eng_ing_form[j] = input_buffer[i];
    i++;  j++;
  }
  for (; j < 19; j++)
    dict_verb.eng_ing_form[j] = '\x00';  /* null out rest of word */
  i++;        /*  space past next comma  */
  j = 0;      /*  reset field count      */
  while (input_buffer[i] != ',')
  {
    if (j == 18)
    {
      cout << "\n" << "English '-s' form in line " << record
           << " more than eighteen letters long";
      rc = 12;
      goto cleanup;
    }
    dict_verb.eng_s_form[j] = input_buffer[i];
    i++;  j++;
  }
  for (; j < 19; j++)
    dict_verb.eng_s_form[j] = '\x00';  /* null out rest of word */
  i++;        /*  space past next comma  */
  j = 0;      /*  reset field count      */
  while (input_buffer[i] != ',')
  {
    if (j == 18)
    {
      cout << "\n" << "English past form in line " << record
           << " more than nineteen letters long";
      rc = 12;
      goto cleanup;
    }
    dict_verb.eng_past[j] = input_buffer[i];
    i++;  j++;
  }
  for (; j < 19; j++)
    dict_verb.eng_past[j] = '\x00';  /* null out rest of word */
  i++;        /*  space past next comma  */
  j = 0;      /*  reset field count      */
  while (input_buffer[i] != ';')
  {
    if (j == 18)
    {
      cout << "\n" << "English participle form in line " << record
           << " more than nineteen letters long";
      rc = 12;
      goto cleanup;
    }
    dict_verb.eng_participle[j] = input_buffer[i];
    i++;  j++;
  }
  for (; j < 19; j++)
    dict_verb.eng_participle[j] = '\x00';  /* null out rest of word */
  dict_verb.filler2 = ' ';
  fwrite (&dict_verb, 120, 1, dictionary);
  goto loop;    /*  go back to read next record  */
cleanup:
  fclose (source);
cleanup2:
  fclose (dictionary);
  if (rc > 0)  return rc;
//* Update verb and total word counts in control file
  ctl_file = fopen ("SWAHILI.CTL","rb");
  if (ctl_file == NULL)
  {
    cout << "\n" << "SWAHILI.CTL does not exist: run NOUNS first";
    cout << "\n" << "then run VERBS and ADJETC to complete file";
    return 12;
  }
  cout << "\n" << "Updating control file - SWAHILI.CTL";
  fread (&control_data, 32, 1, ctl_file);
  fclose (ctl_file);
  control_data.tot_swahili_words = record +
    control_data.noun_dict_entries +
    control_data.adjetc_dict_entries;
  control_data.verb_dict_entries = record;
  ctl_file = fopen ("SWAHILI.CTL","wb");
  fwrite (&control_data, 32, 1, ctl_file);
  fclose (ctl_file);
  return rc;
}
